@@ -8,18 +8,17 @@ PACKAGE_LOGGER = "aws_lambda_powertools"
 
 def copy_config_to_registered_loggers(
     source_logger: Logger,
-    log_level: Optional[str] = None,
+    log_level: Optional[Union[int, str]] = None,
     exclude: Optional[Set[str]] = None,
     include: Optional[Set[str]] = None,
 ) -> None:
-
     """Copies source Logger level and handler to all registered loggers for consistent formatting.
 
     Parameters
     ----------
     source_logger : Logger
         Powertools Logger to copy configuration from
-    log_level : str, optional
+    log_level : Union[int, str], optional
         Logging level to set to registered loggers, by default uses source_logger logging level
     include : Optional[Set[str]], optional
         List of logger names to include, by default all registered loggers are included
@@ -81,6 +80,8 @@ def _configure_logger(source_logger: Logger, logger: logging.Logger, level: Unio
     logger.handlers = []
     logger.setLevel(level)
     logger.propagate = False  # ensure we don't propagate logs to existing loggers, #1073
+    source_logger.append_keys(name="%(name)s")  # include logger name, see #1267
+
     source_logger.debug(f"Logger {logger} reconfigured to use logging level {level}")
     for source_handler in source_logger.handlers:
         logger.addHandler(source_handler)
