@@ -3,19 +3,22 @@
 """
 Batch processing utilities
 """
+import asyncio
 import copy
 import inspect
 import logging
+import os
 import sys
 from abc import ABC, abstractmethod
 from enum import Enum
-from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union, overload
+from typing import Any, Callable, Dict, List, Optional, Tuple, Union, overload
 
-from aws_lambda_powertools.middleware_factory import lambda_handler_decorator
+from aws_lambda_powertools.shared import constants
 from aws_lambda_powertools.utilities.batch.exceptions import (
     BatchProcessingError,
     ExceptionInfo,
 )
+from aws_lambda_powertools.utilities.batch.types import BatchTypeModels
 from aws_lambda_powertools.utilities.data_classes.dynamo_db_stream_event import (
     DynamoDBRecord,
 )
@@ -23,6 +26,7 @@ from aws_lambda_powertools.utilities.data_classes.kinesis_stream_event import (
     KinesisStreamRecord,
 )
 from aws_lambda_powertools.utilities.data_classes.sqs_event import SQSRecord
+from aws_lambda_powertools.utilities.parser import ValidationError
 from aws_lambda_powertools.utilities.typing import LambdaContext
 
 logger = logging.getLogger(__name__)
@@ -34,24 +38,6 @@ class EventType(Enum):
     DynamoDBStreams = "DynamoDBStreams"
 
 
-#
-# type specifics
-#
-has_pydantic = "pydantic" in sys.modules
-
-# For IntelliSense and Mypy to work, we need to account for possible SQS, Kinesis and DynamoDB subclasses
-# We need them as subclasses as we must access their message ID or sequence number metadata via dot notation
-if has_pydantic:
-    from aws_lambda_powertools.utilities.parser.models import DynamoDBStreamRecordModel
-    from aws_lambda_powertools.utilities.parser.models import (
-        KinesisDataStreamRecord as KinesisDataStreamRecordModel,
-    )
-    from aws_lambda_powertools.utilities.parser.models import SqsRecordModel
-
-    BatchTypeModels = Optional[
-        Union[Type[SqsRecordModel], Type[DynamoDBStreamRecordModel], Type[KinesisDataStreamRecordModel]]
-    ]
-
 # When using processor with default arguments, records will carry EventSourceDataClassTypes
 # and depending on what EventType it's passed it'll correctly map to the right record
 # When using Pydantic Models, it'll accept any subclass from SQS, DynamoDB and Kinesis
@@ -100,6 +86,49 @@ class BasePartialProcessor(ABC):
         """
         return [self._process_record(record) for record in self.records]
 
+    @abstractmethod
+    async def _async_process_record(self, record: dict):
+        """
+        Async process record with handler.
+        """
+        raise NotImplementedError()
+
+    def async_process(self) -> List[Tuple]:
+        """
+        Async call instance's handler for each record.
+
+        Note
+        ----
+
+        We keep the outer function synchronous to prevent making Lambda handler async, so to not impact
+        customers' existing middlewares. Instead, we create an async closure to handle asynchrony.
+
+        We also handle edge cases like Lambda container thaw by getting an existing or creating an event loop.
+
+        See: https://docs.aws.amazon.com/lambda/latest/dg/lambda-runtime-environment.html#runtimes-lifecycle-shutdown
+        """
+
+        async def async_process_closure():
+            return list(await asyncio.gather(*[self._async_process_record(record) for record in self.records]))
+
+        # WARNING
+        # Do not use "asyncio.run(async_process())" due to Lambda container thaws/freeze, otherwise we might get "Event Loop is closed" # noqa: E501
+        # Instead, get_event_loop() can also create one if a previous was erroneously closed
+        # Mangum library does this as well. It's battle tested with other popular async-only frameworks like FastAPI
+        # https://github.com/jordaneremieff/mangum/discussions/256#discussioncomment-2638946
+        # https://github.com/jordaneremieff/mangum/blob/b85cd4a97f8ddd56094ccc540ca7156c76081745/mangum/protocols/http.py#L44
+
+        # Let's prime the coroutine and decide
+        # whether we create an event loop (Lambda) or schedule it as usual (non-Lambda)
+        coro = async_process_closure()
+        if os.getenv(constants.LAMBDA_TASK_ROOT_ENV):
+            loop = asyncio.get_event_loop()  # NOTE: this might return an error starting in Python 3.12 in a few years
+            task_instance = loop.create_task(coro)
+            return loop.run_until_complete(task_instance)
+
+        # Non-Lambda environment, run coroutine as usual
+        return asyncio.run(coro)
+
     def __enter__(self):
         self._prepare()
         return self
@@ -191,56 +220,153 @@ class BasePartialProcessor(ABC):
         return entry
 
 
-@lambda_handler_decorator
-def batch_processor(
-    handler: Callable, event: Dict, context: LambdaContext, record_handler: Callable, processor: BasePartialProcessor
-):
-    """
-    Middleware to handle batch event processing
+class BasePartialBatchProcessor(BasePartialProcessor):  # noqa
+    DEFAULT_RESPONSE: Dict[str, List[Optional[dict]]] = {"batchItemFailures": []}
 
-    Parameters
-    ----------
-    handler: Callable
-        Lambda's handler
-    event: Dict
-        Lambda's Event
-    context: LambdaContext
-        Lambda's Context
-    record_handler: Callable
-        Callable to process each record from the batch
-    processor: BasePartialProcessor
-        Batch Processor to handle partial failure cases
+    def __init__(self, event_type: EventType, model: Optional["BatchTypeModels"] = None):
+        """Process batch and partially report failed items
 
-    Examples
-    --------
-    **Processes Lambda's event with a BasePartialProcessor**
+        Parameters
+        ----------
+        event_type: EventType
+            Whether this is a SQS, DynamoDB Streams, or Kinesis Data Stream event
+        model: Optional["BatchTypeModels"]
+            Parser's data model using either SqsRecordModel, DynamoDBStreamRecordModel, KinesisDataStreamRecord
 
-        >>> from aws_lambda_powertools.utilities.batch import batch_processor, BatchProcessor
-        >>>
-        >>> def record_handler(record):
-        >>>     return record["body"]
-        >>>
-        >>> @batch_processor(record_handler=record_handler, processor=BatchProcessor())
-        >>> def handler(event, context):
-        >>>     return {"StatusCode": 200}
+        Exceptions
+        ----------
+        BatchProcessingError
+            Raised when the entire batch has failed processing
+        """
+        self.event_type = event_type
+        self.model = model
+        self.batch_response = copy.deepcopy(self.DEFAULT_RESPONSE)
+        self._COLLECTOR_MAPPING = {
+            EventType.SQS: self._collect_sqs_failures,
+            EventType.KinesisDataStreams: self._collect_kinesis_failures,
+            EventType.DynamoDBStreams: self._collect_dynamodb_failures,
+        }
+        self._DATA_CLASS_MAPPING = {
+            EventType.SQS: SQSRecord,
+            EventType.KinesisDataStreams: KinesisStreamRecord,
+            EventType.DynamoDBStreams: DynamoDBRecord,
+        }
 
-    Limitations
-    -----------
-    * Async batch processors
+        super().__init__()
 
-    """
-    records = event["Records"]
+    def response(self):
+        """Batch items that failed processing, if any"""
+        return self.batch_response
 
-    with processor(records, record_handler, lambda_context=context):
-        processor.process()
+    def _prepare(self):
+        """
+        Remove results from previous execution.
+        """
+        self.success_messages.clear()
+        self.fail_messages.clear()
+        self.exceptions.clear()
+        self.batch_response = copy.deepcopy(self.DEFAULT_RESPONSE)
 
-    return handler(event, context)
+    def _clean(self):
+        """
+        Report messages to be deleted in case of partial failure.
+        """
+
+        if not self._has_messages_to_report():
+            return
+
+        if self._entire_batch_failed():
+            raise BatchProcessingError(
+                msg=f"All records failed processing. {len(self.exceptions)} individual errors logged "
+                f"separately below.",
+                child_exceptions=self.exceptions,
+            )
+
+        messages = self._get_messages_to_report()
+        self.batch_response = {"batchItemFailures": messages}
+
+    def _has_messages_to_report(self) -> bool:
+        if self.fail_messages:
+            return True
+
+        logger.debug(f"All {len(self.success_messages)} records successfully processed")
+        return False
+
+    def _entire_batch_failed(self) -> bool:
+        return len(self.exceptions) == len(self.records)
+
+    def _get_messages_to_report(self) -> List[Dict[str, str]]:
+        """
+        Format messages to use in batch deletion
+        """
+        return self._COLLECTOR_MAPPING[self.event_type]()
+
+    # Event Source Data Classes follow python idioms for fields
+    # while Parser/Pydantic follows the event field names to the latter
+    def _collect_sqs_failures(self):
+        failures = []
+        for msg in self.fail_messages:
+            # If a message failed due to model validation (e.g., poison pill)
+            # we convert to an event source data class...but self.model is still true
+            # therefore, we do an additional check on whether the failed message is still a model
+            # see https://github.com/awslabs/aws-lambda-powertools-python/issues/2091
+            if self.model and getattr(msg, "parse_obj", None):
+                msg_id = msg.messageId
+            else:
+                msg_id = msg.message_id
+            failures.append({"itemIdentifier": msg_id})
+        return failures
+
+    def _collect_kinesis_failures(self):
+        failures = []
+        for msg in self.fail_messages:
+            # # see https://github.com/awslabs/aws-lambda-powertools-python/issues/2091
+            if self.model and getattr(msg, "parse_obj", None):
+                msg_id = msg.kinesis.sequenceNumber
+            else:
+                msg_id = msg.kinesis.sequence_number
+            failures.append({"itemIdentifier": msg_id})
+        return failures
+
+    def _collect_dynamodb_failures(self):
+        failures = []
+        for msg in self.fail_messages:
+            # see https://github.com/awslabs/aws-lambda-powertools-python/issues/2091
+            if self.model and getattr(msg, "parse_obj", None):
+                msg_id = msg.dynamodb.SequenceNumber
+            else:
+                msg_id = msg.dynamodb.sequence_number
+            failures.append({"itemIdentifier": msg_id})
+        return failures
+
+    @overload
+    def _to_batch_type(self, record: dict, event_type: EventType, model: "BatchTypeModels") -> "BatchTypeModels":
+        ...  # pragma: no cover
+
+    @overload
+    def _to_batch_type(self, record: dict, event_type: EventType) -> EventSourceDataClassTypes:
+        ...  # pragma: no cover
+
+    def _to_batch_type(self, record: dict, event_type: EventType, model: Optional["BatchTypeModels"] = None):
+        if model is not None:
+            return model.parse_obj(record)
+        return self._DATA_CLASS_MAPPING[event_type](record)
+
+    def _register_model_validation_error_record(self, record: dict):
+        """Convert and register failure due to poison pills where model failed validation early"""
+        # Parser will fail validation if record is a poison pill (malformed input)
+        # this means we can't collect the message id if we try transforming again
+        # so we convert into to the equivalent batch type model (e.g., SQS, Kinesis, DynamoDB Stream)
+        # and downstream we can correctly collect the correct message id identifier and make the failed record available
+        # see https://github.com/awslabs/aws-lambda-powertools-python/issues/2091
+        logger.debug("Record cannot be converted to customer's model; converting without model")
+        failed_record: "EventSourceDataClassTypes" = self._to_batch_type(record=record, event_type=self.event_type)
+        return self.failure_handler(record=failed_record, exception=sys.exc_info())
 
 
-class BatchProcessor(BasePartialProcessor):
+class BatchProcessor(BasePartialBatchProcessor):  # Keep old name for compatibility
     """Process native partial responses from SQS, Kinesis Data Streams, and DynamoDB.
 
-
     Example
     -------
 
@@ -303,7 +429,6 @@ class BatchProcessor(BasePartialProcessor):
         return processor.response()
     ```
 
-
     ## Process batch triggered by DynamoDB Data Streams
 
     ```python
@@ -344,53 +469,14 @@ class BatchProcessor(BasePartialProcessor):
     ------
     BatchProcessingError
         When all batch records fail processing
+
+    Limitations
+    -----------
+    * Async record handler not supported, use AsyncBatchProcessor instead.
     """
 
-    DEFAULT_RESPONSE: Dict[str, List[Optional[dict]]] = {"batchItemFailures": []}
-
-    def __init__(self, event_type: EventType, model: Optional["BatchTypeModels"] = None):
-        """Process batch and partially report failed items
-
-        Parameters
-        ----------
-        event_type: EventType
-            Whether this is a SQS, DynamoDB Streams, or Kinesis Data Stream event
-        model: Optional["BatchTypeModels"]
-            Parser's data model using either SqsRecordModel, DynamoDBStreamRecordModel, KinesisDataStreamRecord
-
-        Exceptions
-        ----------
-        BatchProcessingError
-            Raised when the entire batch has failed processing
-        """
-        self.event_type = event_type
-        self.model = model
-        self.batch_response = copy.deepcopy(self.DEFAULT_RESPONSE)
-        self._COLLECTOR_MAPPING = {
-            EventType.SQS: self._collect_sqs_failures,
-            EventType.KinesisDataStreams: self._collect_kinesis_failures,
-            EventType.DynamoDBStreams: self._collect_dynamodb_failures,
-        }
-        self._DATA_CLASS_MAPPING = {
-            EventType.SQS: SQSRecord,
-            EventType.KinesisDataStreams: KinesisStreamRecord,
-            EventType.DynamoDBStreams: DynamoDBRecord,
-        }
-
-        super().__init__()
-
-    def response(self):
-        """Batch items that failed processing, if any"""
-        return self.batch_response
-
-    def _prepare(self):
-        """
-        Remove results from previous execution.
-        """
-        self.success_messages.clear()
-        self.fail_messages.clear()
-        self.exceptions.clear()
-        self.batch_response = copy.deepcopy(self.DEFAULT_RESPONSE)
+    async def _async_process_record(self, record: dict):
+        raise NotImplementedError()
 
     def _process_record(self, record: dict) -> Union[SuccessResponse, FailureResponse]:
         """
@@ -401,83 +487,154 @@ class BatchProcessor(BasePartialProcessor):
         record: dict
             A batch record to be processed.
         """
-        data = self._to_batch_type(record=record, event_type=self.event_type, model=self.model)
+        data: Optional["BatchTypeModels"] = None
         try:
+            data = self._to_batch_type(record=record, event_type=self.event_type, model=self.model)
             if self._handler_accepts_lambda_context:
                 result = self.handler(record=data, lambda_context=self.lambda_context)
             else:
                 result = self.handler(record=data)
 
             return self.success_handler(record=record, result=result)
+        except ValidationError:
+            return self._register_model_validation_error_record(record)
         except Exception:
             return self.failure_handler(record=data, exception=sys.exc_info())
 
-    def _clean(self):
+
+class AsyncBatchProcessor(BasePartialBatchProcessor):
+    """Process native partial responses from SQS, Kinesis Data Streams, and DynamoDB asynchronously.
+
+    Example
+    -------
+
+    ## Process batch triggered by SQS
+
+    ```python
+    import json
+
+    from aws_lambda_powertools import Logger, Tracer
+    from aws_lambda_powertools.utilities.batch import BatchProcessor, EventType, batch_processor
+    from aws_lambda_powertools.utilities.data_classes.sqs_event import SQSRecord
+    from aws_lambda_powertools.utilities.typing import LambdaContext
+
+
+    processor = BatchProcessor(event_type=EventType.SQS)
+    tracer = Tracer()
+    logger = Logger()
+
+
+    @tracer.capture_method
+    async def record_handler(record: SQSRecord):
+        payload: str = record.body
+        if payload:
+            item: dict = json.loads(payload)
+        ...
+
+    @logger.inject_lambda_context
+    @tracer.capture_lambda_handler
+    @batch_processor(record_handler=record_handler, processor=processor)
+    def lambda_handler(event, context: LambdaContext):
+        return processor.response()
+    ```
+
+    ## Process batch triggered by Kinesis Data Streams
+
+    ```python
+    import json
+
+    from aws_lambda_powertools import Logger, Tracer
+    from aws_lambda_powertools.utilities.batch import BatchProcessor, EventType, batch_processor
+    from aws_lambda_powertools.utilities.data_classes.kinesis_stream_event import KinesisStreamRecord
+    from aws_lambda_powertools.utilities.typing import LambdaContext
+
+
+    processor = BatchProcessor(event_type=EventType.KinesisDataStreams)
+    tracer = Tracer()
+    logger = Logger()
+
+
+    @tracer.capture_method
+    async def record_handler(record: KinesisStreamRecord):
+        logger.info(record.kinesis.data_as_text)
+        payload: dict = record.kinesis.data_as_json()
+        ...
+
+    @logger.inject_lambda_context
+    @tracer.capture_lambda_handler
+    @batch_processor(record_handler=record_handler, processor=processor)
+    def lambda_handler(event, context: LambdaContext):
+        return processor.response()
+    ```
+
+    ## Process batch triggered by DynamoDB Data Streams
+
+    ```python
+    import json
+
+    from aws_lambda_powertools import Logger, Tracer
+    from aws_lambda_powertools.utilities.batch import BatchProcessor, EventType, batch_processor
+    from aws_lambda_powertools.utilities.data_classes.dynamo_db_stream_event import DynamoDBRecord
+    from aws_lambda_powertools.utilities.typing import LambdaContext
+
+
+    processor = BatchProcessor(event_type=EventType.DynamoDBStreams)
+    tracer = Tracer()
+    logger = Logger()
+
+
+    @tracer.capture_method
+    async def record_handler(record: DynamoDBRecord):
+        logger.info(record.dynamodb.new_image)
+        payload: dict = json.loads(record.dynamodb.new_image.get("item"))
+        # alternatively:
+        # changes: Dict[str, Any] = record.dynamodb.new_image  # noqa: E800
+        # payload = change.get("Message") -> "<payload>"
+        ...
+
+    @logger.inject_lambda_context
+    @tracer.capture_lambda_handler
+    def lambda_handler(event, context: LambdaContext):
+        batch = event["Records"]
+        with processor(records=batch, processor=processor):
+            processed_messages = processor.process() # kick off processing, return list[tuple]
+
+        return processor.response()
+    ```
+
+
+    Raises
+    ------
+    BatchProcessingError
+        When all batch records fail processing
+
+    Limitations
+    -----------
+    * Sync record handler not supported, use BatchProcessor instead.
+    """
+
+    def _process_record(self, record: dict):
+        raise NotImplementedError()
+
+    async def _async_process_record(self, record: dict) -> Union[SuccessResponse, FailureResponse]:
         """
-        Report messages to be deleted in case of partial failure.
+        Process a record with instance's handler
+
+        Parameters
+        ----------
+        record: dict
+            A batch record to be processed.
         """
+        data: Optional["BatchTypeModels"] = None
+        try:
+            data = self._to_batch_type(record=record, event_type=self.event_type, model=self.model)
+            if self._handler_accepts_lambda_context:
+                result = await self.handler(record=data, lambda_context=self.lambda_context)
+            else:
+                result = await self.handler(record=data)
 
-        if not self._has_messages_to_report():
-            return
-
-        if self._entire_batch_failed():
-            raise BatchProcessingError(
-                msg=f"All records failed processing. {len(self.exceptions)} individual errors logged "
-                f"separately below.",
-                child_exceptions=self.exceptions,
-            )
-
-        messages = self._get_messages_to_report()
-        self.batch_response = {"batchItemFailures": messages}
-
-    def _has_messages_to_report(self) -> bool:
-        if self.fail_messages:
-            return True
-
-        logger.debug(f"All {len(self.success_messages)} records successfully processed")
-        return False
-
-    def _entire_batch_failed(self) -> bool:
-        return len(self.exceptions) == len(self.records)
-
-    def _get_messages_to_report(self) -> List[Dict[str, str]]:
-        """
-        Format messages to use in batch deletion
-        """
-        return self._COLLECTOR_MAPPING[self.event_type]()
-
-    # Event Source Data Classes follow python idioms for fields
-    # while Parser/Pydantic follows the event field names to the latter
-    def _collect_sqs_failures(self):
-        failures = []
-        for msg in self.fail_messages:
-            msg_id = msg.messageId if self.model else msg.message_id
-            failures.append({"itemIdentifier": msg_id})
-        return failures
-
-    def _collect_kinesis_failures(self):
-        failures = []
-        for msg in self.fail_messages:
-            msg_id = msg.kinesis.sequenceNumber if self.model else msg.kinesis.sequence_number
-            failures.append({"itemIdentifier": msg_id})
-        return failures
-
-    def _collect_dynamodb_failures(self):
-        failures = []
-        for msg in self.fail_messages:
-            msg_id = msg.dynamodb.SequenceNumber if self.model else msg.dynamodb.sequence_number
-            failures.append({"itemIdentifier": msg_id})
-        return failures
-
-    @overload
-    def _to_batch_type(self, record: dict, event_type: EventType, model: "BatchTypeModels") -> "BatchTypeModels":
-        ...  # pragma: no cover
-
-    @overload
-    def _to_batch_type(self, record: dict, event_type: EventType) -> EventSourceDataClassTypes:
-        ...  # pragma: no cover
-
-    def _to_batch_type(self, record: dict, event_type: EventType, model: Optional["BatchTypeModels"] = None):
-        if model is not None:
-            return model.parse_obj(record)
-        return self._DATA_CLASS_MAPPING[event_type](record)
+            return self.success_handler(record=record, result=result)
+        except ValidationError:
+            return self._register_model_validation_error_record(record)
+        except Exception:
+            return self.failure_handler(record=data, exception=sys.exc_info())
